"""Import net classes."""

from ansys.edb.core.primitive.primitive import (
    BackDrillType,
    BoardBendDef,
    Bondwire,
    BondwireCrossSectionType,
    BondwireType,
    Circle,
    PadstackInstance,
    Path,
    PathCornerType,
    PathEndCapType,
    Polygon,
    Primitive,
    PrimitiveType,
    Rectangle,
    RectangleRepresentationType,
    Text,
)
